@@ -1,6 +1,7 @@
 import typer
 
 from belay import Device
+from belay.cli.clean import clean
 from belay.cli.common import help_password, help_port
 from belay.cli.exec import exec
 from belay.cli.identify import identify
@@ -11,10 +12,11 @@ from belay.cli.sync import sync
 from belay.cli.update import update
 
 app = typer.Typer()
-app.command()(sync)
-app.command()(run)
+app.command()(clean)
 app.command()(exec)
-app.command()(info)
 app.command()(identify)
-app.command()(update)
+app.command()(info)
 app.command()(install)
+app.command()(run)
+app.command()(sync)
+app.command()(update)
