@@ -119,6 +119,7 @@ class _TaskExecuter(_Executer):
         /,
         minify: bool = True,
         register: bool = True,
+        record: bool = False,
     ) -> BelayCallable:
         """Decorator that send code to device that executes when decorated function is called on-host.
 
@@ -132,6 +133,10 @@ class _TaskExecuter(_Executer):
         register: bool
             Assign an attribute to ``self`` with same name as ``f``.
             Defaults to ``True``.
+        record: bool
+            Record task execution calls for state-reconstruction if device is accidentally reset.
+            Only recommended for tasks that are called a low amount of times to setup device state.
+            Defaults to ``False``.
 
         Returns
         -------
@@ -156,7 +161,7 @@ class _TaskExecuter(_Executer):
             cmd = f"_belay_{name}(*{repr(args)}, **{repr(kwargs)})"
 
             return self._belay_device._traceback_execute(
-                src_file, src_lineno, name, cmd, record=False
+                src_file, src_lineno, name, cmd, record=record
             )
 
         @wraps(f)
@@ -177,6 +182,10 @@ class _TaskExecuter(_Executer):
 
         @wraps(f)
         def gen_executer(*args, **kwargs):
+            if record:
+                raise NotImplementedError(
+                    "Recording of generator tasks is currently not supported."
+                )
             # Step 1: Create the on-device generator
             gen_identifier = _random_python_identifier()
             cmd = f"{gen_identifier} = _belay_{name}(*{repr(args)}, **{repr(kwargs)})"
@@ -184,12 +193,13 @@ class _TaskExecuter(_Executer):
                 src_file, src_lineno, name, cmd, record=False
             )
             # Step 2: Create the host generator that invokes ``next()`` on-device.
-            cmd = f"__belay_gen_next({gen_identifier})"
 
             def gen_inner():
+                send_val = None
                 try:
                     while True:
-                        yield self._belay_device._traceback_execute(
+                        cmd = f"__belay_gen_next({gen_identifier}, {repr(send_val)})"
+                        send_val = yield self._belay_device._traceback_execute(
                             src_file, src_lineno, name, cmd, record=False
                         )
                 except StopIteration:
@@ -231,6 +241,7 @@ class _ThreadExecuter(_Executer):
         /,
         minify: bool = True,
         register: bool = True,
+        record: bool = True,
     ) -> Callable[..., None]:
         """Decorator that send code to device that spawns a thread when executed.
 
@@ -244,6 +255,9 @@ class _ThreadExecuter(_Executer):
         register: bool
             Assign an attribute to ``self`` with same name as ``f``.
             Defaults to ``True``.
+        record: bool
+            Record thread execution calls for state-reconstruction if device is accidentally reset.
+            Defaults to ``True``.
 
         Returns
         -------
@@ -266,7 +280,7 @@ class _ThreadExecuter(_Executer):
         def executer(*args, **kwargs):
             cmd = f"import _thread; _thread.start_new_thread({name}, {repr(args)}, {repr(kwargs)})"
             self._belay_device._traceback_execute(
-                src_file, src_lineno, name, cmd, record=False
+                src_file, src_lineno, name, cmd, record=record
             )
 
         @wraps(f)
@@ -331,7 +345,7 @@ def _preprocess_keep(
 ) -> list:
     if keep is None:
         if dst == "/":
-            keep = ["boot.py", "webrepl_cfg.py"]
+            keep = ["boot.py", "webrepl_cfg.py", "lib"]
         else:
             keep = []
     elif isinstance(keep, str):
@@ -625,7 +639,7 @@ class Device:
             Do NOT delete these file(s) on-device if not present in ``folder``.
             If ``true``, don't delete any files on device.
             If ``false``, delete all unsynced files (same as passing ``[]``).
-            If ``dst is None``, defaults to ``["boot.py", "webrepl_cfg.py"]``.
+            If ``dst is None``, defaults to ``["boot.py", "webrepl_cfg.py", "lib"]``.
         ignore: None | str | list
             Git's wildmatch patterns to NOT sync to the device.
             Defaults to ``["*.pyc", "__pycache__", ".DS_Store", ".pytest_cache"]``.
@@ -676,12 +690,6 @@ class Device:
         keep = _preprocess_keep(keep, dst)
         ignore = _preprocess_ignore(ignore)
 
-        if keep_all:
-            # Don't build up the device of files, we won't be deleting anything
-            self("del __belay_fs")
-        else:
-            self(f"__belay_fs({repr(dst)}); all_dirs.sort(); del __belay_fs")
-
         src_files, src_dirs, dst_files = _discover_files_dirs(dst, folder, ignore)
 
         if mpy_cross_binary:
@@ -692,11 +700,12 @@ class Device:
         dst_files = [str(dst_file) for dst_file in dst_files]
         dst_dirs = _generate_dst_dirs(dst, folder, src_dirs)
 
-        if not keep_all:
-            # prevent keep duplicates in the concat'd file list
-            keep = [x for x in keep if x not in dst_files]
-            if dst_files + keep:
-                self(f"for x in {repr(dst_files + keep)}:\n all_files.discard(x)")
+        if keep_all:
+            self("del __belay_del_fs")
+        else:
+            self(
+                f"__belay_del_fs({repr(dst)}, {repr(set(keep + dst_files))}); del __belay_del_fs"
+            )
 
         # Try and make all remote dirs
         if dst_dirs:
@@ -744,7 +753,6 @@ class Device:
         # Remove all the files and directories that did not exist in local filesystem.
         if progress_update:
             progress_update(description="Cleaning up...")
-        self._exec_snippet("sync_end")
 
     def __enter__(self):
         return self
